@@ -142,6 +142,13 @@ class Matchup(NexusCalcSparkHandler):
                            "If the number of primary matches is greater than this limit, the service will respond with "
                            "(HTTP 202: Accepted) and an empty response body. A value of 0 means return all results. "
                            "Default: 500"
+        },
+        "prioritizeDistance": {
+            "name": "Prioritize distance",
+            "type": "boolean",
+            "description": "If true, prioritize distance over time when computing matches. If false, prioritize time over "
+                           "distance. This is only relevant if matchOnce=true, because otherwise all matches will be "
+                           "included so long as they fit within the user-provided tolerances. Default is true."
         }
     }
     singleton = True
@@ -169,10 +176,11 @@ class Matchup(NexusCalcSparkHandler):
             raise NexusProcessingException(reason="'secondary' argument is required", code=400)
 
         parameter_s = request.get_argument('parameter')
-        insitu_params = get_insitu_params(insitu_schema.get())
-        if parameter_s and parameter_s not in insitu_params:
-            raise NexusProcessingException(
-                reason=f"Parameter {parameter_s} not supported. Must be one of {insitu_params}", code=400)
+        if parameter_s:
+            insitu_params = get_insitu_params(insitu_schema.get())
+            if parameter_s not in insitu_params:
+                raise NexusProcessingException(
+                    reason=f"Parameter {parameter_s} not supported. Must be one of {insitu_params}", code=400)
 
         try:
             start_time = request.get_start_datetime()
@@ -213,10 +221,13 @@ class Matchup(NexusCalcSparkHandler):
         start_seconds_from_epoch = int((start_time - EPOCH).total_seconds())
         end_seconds_from_epoch = int((end_time - EPOCH).total_seconds())
 
+        prioritize_distance = request.get_boolean_arg("prioritizeDistance", default=True)
+
+
         return bounding_polygon, primary_ds_name, secondary_ds_names, parameter_s, \
                start_time, start_seconds_from_epoch, end_time, end_seconds_from_epoch, \
                depth_min, depth_max, time_tolerance, radius_tolerance, \
-               platforms, match_once, result_size_limit
+               platforms, match_once, result_size_limit, prioritize_distance
 
     def calc(self, request, **args):
         start = datetime.utcnow()
@@ -224,7 +235,7 @@ class Matchup(NexusCalcSparkHandler):
         bounding_polygon, primary_ds_name, secondary_ds_names, parameter_s, \
         start_time, start_seconds_from_epoch, end_time, end_seconds_from_epoch, \
         depth_min, depth_max, time_tolerance, radius_tolerance, \
-        platforms, match_once, result_size_limit = self.parse_arguments(request)
+        platforms, match_once, result_size_limit, prioritize_distance = self.parse_arguments(request)
 
         with ResultsStorage(self.config) as resultsStorage:
 
@@ -245,7 +256,8 @@ class Matchup(NexusCalcSparkHandler):
         try:
             spark_result = spark_matchup_driver(tile_ids, wkt.dumps(bounding_polygon), primary_ds_name,
                                                 secondary_ds_names, parameter_s, depth_min, depth_max, time_tolerance,
-                                                radius_tolerance, platforms, match_once, self.tile_service_factory, sc=self._sc)
+                                                radius_tolerance, platforms, match_once, self.tile_service_factory,
+                                                prioritize_distance, sc=self._sc)
         except Exception as e:
             self.log.exception(e)
             raise NexusProcessingException(reason="An unknown error occurred while computing matches", code=500)
@@ -292,9 +304,11 @@ class Matchup(NexusCalcSparkHandler):
         # Get only the first "result_size_limit" results
         # '0' means returns everything
         if result_size_limit > 0:
-            matches = matches[0:result_size_limit]
+            return_matches = matches[0:result_size_limit]
+        else:
+            return_matches = matches
 
-        result = DomsQueryResults(results=matches, args=args,
+        result = DomsQueryResults(results=return_matches, args=args,
                                   details=details, bounds=None,
                                   count=len(matches), computeOptions=None,
                                   executionId=execution_id)
@@ -367,9 +381,17 @@ class DomsPoint(object):
         self.device = None
         self.file_url = None
 
+        self.__id = id(self)
+
     def __repr__(self):
         return str(self.__dict__)
 
+    def __eq__(self, other):
+        return isinstance(other, DomsPoint) and other.__id == self.__id
+
+    def __hash__(self):
+        return hash(self.data_id) if self.data_id else id(self)
+
     @staticmethod
     def _variables_to_device(variables):
         """
@@ -555,7 +577,7 @@ DRIVER_LOCK = Lock()
 
 
 def spark_matchup_driver(tile_ids, bounding_wkt, primary_ds_name, secondary_ds_names, parameter, depth_min, depth_max,
-                         time_tolerance, radius_tolerance, platforms, match_once, tile_service_factory, sc=None):
+                         time_tolerance, radius_tolerance, platforms, match_once, tile_service_factory, prioritize_distance=True, sc=None):
     from functools import partial
 
     with DRIVER_LOCK:
@@ -602,12 +624,14 @@ def spark_matchup_driver(tile_ids, bounding_wkt, primary_ds_name, secondary_ds_n
         # Method used for calculating the distance between 2 DomsPoints
         from pyproj import Geod
 
-        def dist(primary, matchup):
+        def dist(primary, matchup, prioritize_distance):
             wgs84_geod = Geod(ellps='WGS84')
             lat1, lon1 = (primary.latitude, primary.longitude)
             lat2, lon2 = (matchup.latitude, matchup.longitude)
             az12, az21, distance = wgs84_geod.inv(lon1, lat1, lon2, lat2)
-            return distance, time_dist(primary, matchup)
+            if prioritize_distance:
+                return distance, time_dist(primary, matchup)
+            return time_dist(primary, matchup), distance
 
         def time_dist(primary, matchup):
             primary_time = iso_time_to_epoch(primary.time)
@@ -636,7 +660,11 @@ def spark_matchup_driver(tile_ids, bounding_wkt, primary_ds_name, secondary_ds_n
 
         rdd_filtered = rdd_filtered.map(
             lambda primary_matchup: tuple(
-                [primary_matchup[0], tuple([primary_matchup[1], dist(primary_matchup[0], primary_matchup[1])])]
+                [primary_matchup[0], tuple([primary_matchup[1], dist(
+                    primary_matchup[0],
+                    primary_matchup[1],
+                    prioritize_distance
+                )])]
             )).combineByKey(
                 lambda value: [value],
                 lambda value_list, value: value_list + [value],
@@ -801,31 +829,37 @@ def match_satellite_to_insitu(tile_ids, primary_b, secondary_b, parameter_b, tt_
         polygon = Polygon(
             [(west, south), (east, south), (east, north), (west, north), (west, south)])
 
+        # Find tile IDS from spatial/temporal bounds of partition
         matchup_tiles = tile_service.find_tiles_in_polygon(
             bounding_polygon=polygon,
             ds=secondary_b.value,
             start_time=matchup_min_time,
             end_time=matchup_max_time,
-            fetch_data=True,
+            fl='id',
+            fetch_data=False,
             sort=['tile_min_time_dt asc', 'tile_min_lon asc', 'tile_min_lat asc'],
             rows=5000
         )
 
         # Convert Tile IDS to tiles and convert to UTM lat/lon projection.
         matchup_points = []
+        edge_results = []
         for tile in matchup_tiles:
+            # Retrieve tile data and convert to lat/lon projection
+            tiles = tile_service.find_tile_by_id(tile.tile_id, fetch_data=True)
+            tile = tiles[0]
+
             valid_indices = tile.get_indices()
+
             primary_points = np.array([aeqd_proj(
                 tile.longitudes[aslice[2]],
                 tile.latitudes[aslice[1]]
             ) for aslice in valid_indices])
             matchup_points.extend(primary_points)
+            edge_results.extend(tile_to_edge_points(tile))
 
-        # Convert tiles to 'edge points' which match the format of in-situ edge points.
-        edge_results = []
-        for matchup_tile in matchup_tiles:
-            edge_results.extend(tile_to_edge_points(matchup_tile))
-
+        if len(matchup_points) <= 0:
+            return []
         matchup_points = np.array(matchup_points)
 
     print("%s Time to convert match points for partition %s to %s" % (
